<?php

namespace App\Models\Core;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;
use Kyslik\ColumnSortable\Sortable;
use App\Http\Controllers\AdminControllers\SiteSettingController;

class Booking extends Model
{
    //
    use Sortable;
    public function images(){
        return $this->belongsTo('App\Images');
    }

    public function categories_description(){
        return $this->beliesngsTo('App\Categories_description');
    }

    public $sortable =['booking_id','created_at'];
    public $sortableAs =['name'];

    public function recursivecategories(){
      $items = DB::table('bookings')
           ->select('bookings.booking_id as id', 'bookings.name as name',
            'bookings.mobile as mobile', 'bookings.email as email',  'bookings.address as address','bookings.created_at as date_added',
            'bookings.updated_at as last_modified', 'bookings.appointment_date as appointment_date',
            'bookings.appointment_time as appointment_time','bookings.aprx_cost as aprx_cost','bookings.booking_status as booking_status')
          //->where('booking_status', '1')
          //->orderby('categories_id','ASC')
          ->get();

        

          return  $items;
    }

    public function editrecursivecategories($data){
      $items = DB::table('bookings')
         ->select('bookings.booking_id as id', 'bookings.name as name',
            'bookings.mobile as mobile', 'bookings.email as email',  'bookings.address as address','bookings.created_at as date_added',
            'bookings.updated_at as last_modified', 'bookings.appointment_date as appointment_date',
            'bookings.appointment_time as appointment_time','bookings.aprx_cost as aprx_cost','bookings.booking_status as booking_status')
          ->where('bookings.booking_id','!=', $data->id)
        //  ->where('booking_status', '1')
          //->orderby('categories_id','ASC')
          ->get();

          return  $items;
    }

    public function paginator(){
      $setting = new Setting();
      $myVarsetting = new SiteSettingController($setting);
      $commonsetting = $myVarsetting->commonsetting();

      $categories = Booking::sortable(['booking_id'=>'ASC'])
            ->select('bookings.booking_id as id', 'bookings.name as name',
            'bookings.mobile as mobile', 'bookings.email as email',  'bookings.address as address','bookings.created_at as date_added',
            'bookings.updated_at as last_modified', 'bookings.appointment_date as appointment_date',
            'bookings.appointment_time as appointment_time','bookings.aprx_cost as aprx_cost','bookings.booking_status as booking_status')
           ->groupby('bookings.booking_id')
           ->paginate($commonsetting['pagination']);

            return $categories;
    }

    public function getter($language_id){
      $listingCategories = DB::table('categories')
          ->leftJoin('categories_description','categories_description.categories_id', '=', 'categories.categories_id')
          ->select('categories.categories_id as id', 'categories.categories_image as image',  'categories.created_at as date_added', 'categories.updated_at as last_modified', 'categories_description.categories_name as name', 'categories.categories_slug as slug'
          , 'categories.parent_id')
          ->where('categories_description.language_id','=', $language_id )
          ->where('parent_id','>', '0')
          ->where('categories_status', '1')
          ->get();

       return $listingCategories;
    }

    public function allcategories($language_id){
        $listingCategories = DB::table('categories')
            ->leftJoin('categories_description','categories_description.categories_id', '=', 'categories.categories_id')
            ->select('categories.categories_id as id', 'categories.categories_image as image',  'categories.created_at as date_added', 'categories.updated_at as last_modified', 'categories_description.categories_name as name', 'categories.categories_slug as slug')
            ->where('categories_description.language_id','=', $language_id )
            ->where('categories_status', '1')
            ->get();
  
         return $listingCategories;
      }

    public function filter($data){
      $booking_status = $data['FilterBy'];
      $param = $data['parameter'];

      switch ( $booking_status ) {
          case 'Status':
              $categories = Booking::sortable(['booking_id'=>'ASC'])
                   ->select('bookings.booking_id as id', 'bookings.name as name',
                            'bookings.mobile as mobile', 'bookings.email as email',  'bookings.address as address','bookings.created_at as date_added',
                            'bookings.updated_at as last_modified', 'bookings.appointment_date as appointment_date',
                            'bookings.appointment_time as appointment_time','bookings.aprx_cost as aprx_cost','bookings.booking_status as booking_status')
                        ->where('bookings.booking_status', 'LIKE', '%' . $param . '%')
                        ->groupby('bookings.booking_id')
                        ->paginate(10);

          break;
          case 'Main':

              $categories = Booking::sortable(['booking_id'=>'ASC'])
                   ->select('bookings.booking_id as id', 'bookings.name as name',
                            'bookings.mobile as mobile', 'bookings.email as email',  'bookings.address as address','bookings.created_at as date_added',
                            'bookings.updated_at as last_modified', 'bookings.appointment_date as appointment_date',
                            'bookings.appointment_time as appointment_time','bookings.aprx_cost as aprx_cost','bookings.booking_status as booking_status')
                        ->where('bookings.booking_status', 'LIKE', '%' . $param . '%')
                        ->groupby('bookings.booking_id')
                  ->paginate(10);
              break;
          default:
              $categories = Booking::sortable(['booking_id'=>'ASC'])
                   ->select('bookings.booking_id as id', 'bookings.name as name',
                            'bookings.mobile as mobile', 'bookings.email as email',  'bookings.address as address','bookings.created_at as date_added',
                            'bookings.updated_at as last_modified', 'bookings.appointment_date as appointment_date',
                            'bookings.appointment_time as appointment_time','bookings.aprx_cost as aprx_cost','bookings.booking_status as booking_status')
                        ->where('bookings.booking_status', 'LIKE', '%' . $param . '%')
                        ->groupby('bookings.booking_id')
                  ->paginate(10);
              break;
            }
        return $categories;
    }

    public function insert($name,$mobile,$email,$address,$appointment_date,$appointment_time,$aprx_cost,$date_added,$booking_status){
        $categories = DB::table('bookings')->insertGetId([
            'name'   =>   $name,
            'mobile'   =>   $mobile,
            'email'   =>   $email,
            'address'   =>   $address,
            'appointment_date'   =>   $appointment_date,
            'appointment_time'   =>   $appointment_time,
            'aprx_cost'   =>   $aprx_cost,
            'created_at'		 =>   $date_added,
            'booking_status' => $booking_status
        ]);
        return $categories;
    }



    public function edit($request){
        $category = DB::table('bookings')
            ->select('bookings.booking_id as id', 'bookings.name as name',
            'bookings.mobile as mobile', 'bookings.email as email',  'bookings.address as address','bookings.created_at as date_added',
            'bookings.updated_at as last_modified', 'bookings.appointment_date as appointment_date',
            'bookings.appointment_time as appointment_time','bookings.aprx_cost as aprx_cost','bookings.booking_status as booking_status')
            ->where('bookings.booking_id', $request->id)->get();
        return $category;
    }

    public function updaterecord($booking_id,$name,$mobile,$email,$address,$appointment_date,$appointment_time,$aprx_cost,$last_modified,$booking_status){
      DB::table('bookings')->where('booking_id', $booking_id)->update(
      [
         
           'name'   =>   $name,
            'mobile'   =>   $mobile,
            'email'   =>   $email,
            'address'   =>   $address,
            'appointment_date'   =>   $appointment_date,
            'appointment_time'   =>   $appointment_time,
            'aprx_cost'   =>   $aprx_cost,
            'updated_at'  	     =>   $last_modified,
            'booking_status' => $booking_status
      ]);
     
      
    }




    public function subcategorydes(){
        $categories = DB::table('categories')
            ->leftJoin('categories_description','categories_description.categories_id', '=', 'categories.categories_id')
            ->select('categories.categories_id as mainId', 'categories_description.categories_name as mainName')
            ->where('parent_id', '0')->get();
        return $categories;
    }

    public function editsubcategory($request){
        $editSubCategory = DB::table('bookings')
            ->select('bookings.booking_id as id', 'bookings.name as name',
            'bookings.mobile as mobile', 'bookings.email as email',  'bookings.address as address','bookings.created_at as date_added',
            'bookings.updated_at as last_modified', 'bookings.appointment_date as appointment_date',
            'bookings.appointment_time as appointment_time','bookings.aprx_cost as aprx_cost','bookings.booking_status as booking_status')
            ->where('bookings.booking_id', $request->id)->get();
        return $editSubCategory;
    }

 
    public function deleterecord($request){
        $booking_id = $request->id;
        DB::table('bookings')->where('booking_id', $booking_id)->delete();
        return "success";
    }

}
