<?php

namespace App\Models\Core;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;
use Kyslik\ColumnSortable\Sortable;
use App\Http\Controllers\AdminControllers\SiteSettingController;

class Service extends Model
{
    //
    use Sortable;
    public function images(){
        return $this->belongsTo('App\Images');
    }

    public function categories_description(){
        return $this->beliesngsTo('App\Categories_description');
    }

    public $sortable =['categories_id','created_at'];
    public $sortableAs =['categories_name'];

    public function recursivecategories(){
      $items = DB::table('categories')
          ->leftJoin('categories_description','categories_description.categories_id', '=', 'categories.categories_id')
          ->select('categories.categories_id', 'categories_description.categories_name', 'categories.parent_id')
          ->where('language_id','=', 1)
          ->where('categories_status', '1')
          //->orderby('categories_id','ASC')
          ->get();

          $childs = array();
          foreach($items as $item)
              $childs[$item->parent_id][] = $item;

          foreach($items as $item) if (isset($childs[$item->categories_id]))
              $item->childs = $childs[$item->categories_id];
          if(count($childs)>0){
            $tree = $childs[0];
          }else{
            $tree = $childs;
          }

          return  $tree;
    }

    public function editrecursivecategories($data){
      $items = DB::table('categories')
          ->leftJoin('categories_description','categories_description.categories_id', '=', 'categories.categories_id')
          ->select('categories.categories_id', 'categories_description.categories_name', 'categories.parent_id')
          ->where('language_id','=', 1)
          ->where('categories.categories_id','!=', $data->id)
          ->where('categories_status', '1')
          //->orderby('categories_id','ASC')
          ->get();

          $childs = array();
          foreach($items as $item)
              $childs[$item->parent_id][] = $item;

          foreach($items as $item) if (isset($childs[$item->categories_id]))
              $item->childs = $childs[$item->categories_id];

              if(count($childs)>0){
                  $tree = $childs[0];
                }else{
                  $tree = $childs;
                }
          return  $tree;
    }

    public function paginator(){
      $setting = new Setting();
      $myVarsetting = new SiteSettingController($setting);
      $commonsetting = $myVarsetting->commonsetting();

      $categories =  DB::table('service')
           ->LeftJoin('image_categories as categoryTable', function ($join) {
                $join->on('categoryTable.image_id', '=', 'service.upload_image')
                    ->where(function ($query) {
                        $query->where('categoryTable.image_type', '=', 'THUMBNAIL')
                            ->where('categoryTable.image_type', '!=', 'THUMBNAIL')
                            ->orWhere('categoryTable.image_type', '=', 'ACTUAL');
                    });
            })
            ->LeftJoin('image_categories as iconTable', function ($join) {
                $join->on('iconTable.image_id', '=', 'service.upload_proof')
                    ->where(function ($query) {
                        $query->where('iconTable.image_type', '=', 'THUMBNAIL')
                            ->where('iconTable.image_type', '!=', 'THUMBNAIL')
                            ->orWhere('iconTable.image_type', '=', 'ACTUAL');
                    });
            })
            ->select('service.service_id as id', 'service.upload_image as image','service.upload_proof as icon',  'service.created_at as date_added', 'service.updated_at as last_modified', 'service.name as name','service.email as email',
            'service.shift_start as shift_start','service.shift_end as shift_end','service.qualification as qualification',
            'service.experience as experience','service.mobile as mobile', 'categoryTable.path as imgpath','iconTable.path as iconpath', 
            'service.service_status  as service_status')
         
            
            ->groupby('service.service_id')
            ->paginate($commonsetting['pagination']);

            return $categories;
    }



    
      public function getter(){
        
      $newsCategories = DB::table('category')
          ->leftJoin('service','service.category_id', '=', 'category.category_id')
          ->select('category.category_id as id', 'category.category_name as category_name')
          ->get();

       return $newsCategories;
    }
    public function allcategories($language_id){
        $listingCategories = DB::table('categories')
            ->leftJoin('categories_description','categories_description.categories_id', '=', 'categories.categories_id')
            ->select('categories.categories_id as id', 'categories.categories_image as image',  'categories.created_at as date_added', 'categories.updated_at as last_modified', 'categories_description.categories_name as name', 'categories.categories_slug as slug')
            ->where('categories_description.language_id','=', $language_id )
            ->where('categories_status', '1')
            ->get();
  
         return $listingCategories;
      }

    public function filter($data){
      $name = $data['FilterBy'];
      $param = $data['parameter'];

      switch ( $name ) {
          case 'Name':
              $categories = Categories::sortable(['categories_id'=>'ASC'])
                    ->leftJoin('categories_description','categories_description.categories_id', '=', 'categories.categories_id')
                        ->leftJoin('images','images.id', '=', 'categories.categories_image')
                        ->leftJoin('image_categories as categoryTable','categoryTable.image_id', '=', 'categories.categories_image')
                        ->leftJoin('image_categories as iconTable','iconTable.image_id', '=', 'categories.categories_icon')
                        ->select('categories.categories_id as id', 'categories.categories_image as image',
                        'categories.categories_icon as icon',  'categories.created_at as date_added',
                        'categories.updated_at as last_modified', 'categories_description.categories_name as name',
                        'categories_description.language_id','categoryTable.path as imgpath','iconTable.path as iconpath','categories.categories_status  as categories_status')
                        ->where('categories_description.language_id', '1')
                        ->where(function($query) {
                            $query->where('categoryTable.image_type', '=',  'THUMBNAIL')
                                ->where('categoryTable.image_type','!=',   'THUMBNAIL')
                                ->orWhere('categoryTable.image_type','=',   'ACTUAL')
                                ->where('iconTable.image_type', '=',  'THUMBNAIL')
                                ->where('iconTable.image_type','!=',   'THUMBNAIL')
                                ->orWhere('iconTable.image_type','=',   'ACTUAL');
                        })
                        ->where('categories_description.categories_name', 'LIKE', '%' . $param . '%')
                        ->groupby('categories.categories_id')
                        ->paginate(10);

          break;
          case 'Main':

              $categories = Categories::sortable(['categories_id'=>'ASC'])
                  ->leftJoin('categories_description','categories_description.categories_id', '=', 'categories.categories_id')

                  ->leftJoin('categories as mainCategory','mainCategory.categories_id', '=', 'categories.categories_id')
                  ->leftJoin('categories_description as mainCategoryDesc','mainCategoryDesc.categories_id', '=', 'categories.parent_id')

                  ->leftJoin('image_categories as categoryTable','categoryTable.image_id', '=', 'categories.categories_image')
                  ->leftJoin('image_categories as iconTable','iconTable.image_id', '=', 'categories.categories_icon')

                  ->select(
                      'categories.categories_id as subId',
                      'categories.categories_image as image',
                      'categories.categories_icon as icon',
                      'categories.created_at as date_added',
                      'categories.updated_at as last_modified',
                      'categories_description.categories_name as subCategoryName',
                      'mainCategoryDesc.categories_name as mainCategoryName',
                      'categories_description.language_id','categoryTable.path as imgpath','iconTable.path as iconpath'
                  )
                  ->where('categories.parent_id', '>', '0')
                  ->where('mainCategoryDesc.categories_name', 'LIKE', '%' . $param . '%')
                  ->where('mainCategoryDesc.language_id', '1')
                  ->where('categories_description.language_id', '1')
                  ->groupby('categories.categories_id')
                  ->paginate(10);
              break;
          default:
              $categories = Categories::sortable(['categories_id'=>'ASC'])
                  ->leftJoin('categories_description','categories_description.categories_id', '=', 'categories.categories_id')

                  ->leftJoin('categories as mainCategory','mainCategory.categories_id', '=', 'categories.categories_id')
                  ->leftJoin('categories_description as mainCategoryDesc','mainCategoryDesc.categories_id', '=', 'categories.parent_id')

                  ->leftJoin('image_categories as categoryTable','categoryTable.image_id', '=', 'categories.categories_image')
                  ->leftJoin('image_categories as iconTable','iconTable.image_id', '=', 'categories.categories_icon')

                  ->select(
                      'categories.categories_id as subId',
                      'categories.categories_image as image',
                      'categories.categories_icon as icon',
                      'categories.created_at as date_added',
                      'categories.updated_at as last_modified',
                      'categories_description.categories_name as subCategoryName',
                      'mainCategoryDesc.categories_name as mainCategoryName',
                      'categories_description.language_id','categoryTable.path as imgpath','iconTable.path as iconpath'
                  )
                  ->where('categories.parent_id', '>', '0')->where('mainCategoryDesc.language_id', '1')
                  ->where('categories_description.language_id', '1')
                  ->groupby('categories.categories_id')
                  ->paginate(10);
              break;
            }
        return $categories;
    }


    public function insert($uploadImage,$date_added,$uploadIcon,$service_status,$name,$email,$mobile,$password,$qualification,$experience,$shift_start,$shift_end,$category_id){
        $categories = DB::table('service')->insertGetId([
            'upload_image'   =>   $uploadImage,
            'upload_proof'   =>   $uploadIcon,
            'created_at'		 =>   $date_added,
            'name' => $name,
            'email' => $email,
            'mobile' =>$mobile,
            'password' => $password,
            'qualification' => $qualification,
            'experience' => $experience,
            'shift_start' => $shift_start,
            'shift_end' => $shift_end,
            'service_status' => $service_status,
            'category_id' => $category_id,
        ]);
        return $categories;
    }



    public function edit($request){
        $category = DB::table('service') ->leftJoin('images','images.id', '=', 'service.upload_image')
            ->leftJoin('image_categories as ImageTable','ImageTable.image_id', '=', 'service.upload_image')
            ->leftJoin('image_categories as IconTable','IconTable.image_id', '=', 'service.upload_proof')
            ->select('service.service_id as id','service.service_status as service_status', 'service.name as name','service.mobile as mobile', 'service.upload_image as image','service.upload_proof as icon',  'service.created_at as date_added','service.updated_at as last_modified','service.email as email',
               'service.shift_start as shift_start','service.shift_end as shift_end','service.qualification as qualification','service.experience as experience','service.password as password',
              'ImageTable.path as imagepath','IconTable.path as iconpath')
            ->where('service.service_id', $request->id)->get();
        return $category;
    }
    
           public function listcategory($request){

        $listcategory = DB::table('category')
            ->leftJoin('service','service.category_id', '=', 'category.category_id')
            ->select('service.service_id as id','category.category_id as category_id','category.category_name as category_name')
            ->get();
            

        return $listcategory;
    }
    

    public function updaterecord($service_id,$uploadImage,$last_modified,$uploadIcon,$service_status,$name,$email,$mobile,$password,$shift_start,$shift_end,$qualification,$experience,$category_id){
      DB::table('service')->where('service_id', $service_id)->update(
      [
          'upload_image'   =>   $uploadImage,
          'upload_proof'    =>   $uploadIcon,
          'updated_at'  	     =>   $last_modified,
            'name'              => $name,    
            'email'             => $email,
             'mobile'             => $mobile,
            'password'          =>$password,
            'shift_start'       =>$shift_start,
            'shift_end'         =>$shift_end,
            'qualification'     =>$qualification,
            'experience'        =>$experience,
            'upload_proof'	 =>	  $uploadIcon,
            'service_status' => $service_status,   
            'category_id' =>  $category_id
      ]);
     
      
    }

    public function checkExit($categories_id,$languages_data){
        $checkExist = DB::table('categories_description')->where('categories_id','=',$categories_id)->where('language_id','=',$languages_data->languages_id)->get();
        return $checkExist;
    }

    public function updatedescription($categories_name,$languages_data,$categories_id){
       $category =  DB::table('categories_description')->where('categories_id','=',$categories_id)->where('language_id','=',$languages_data->languages_id)->update([
            'categories_name'  	    		 =>  $categories_name,
        ]);
       return $category;
    }


    public function checkSlug($currentSlug){
        $checkSlug = DB::table('categories')->where('categories_slug',$currentSlug)->get();
        return $checkSlug;
    }

    public function updateSlug($categories_id,$slug){
       $updateSlug = DB::table('categories')->where('categories_id',$categories_id)->update([
            'categories_slug'	 =>   $slug
        ]);
       return $updateSlug;
    }

    public function subcategorydes(){
        $categories = DB::table('categories')
            ->leftJoin('categories_description','categories_description.categories_id', '=', 'categories.categories_id')
            ->select('categories.categories_id as mainId', 'categories_description.categories_name as mainName')
            ->where('parent_id', '0')->get();
        return $categories;
    }

    public function editsubcategory($request){
        $editSubCategory = DB::table('service')
        ->leftJoin('category','service.category_id', '=', 'category.category_id')
            ->leftJoin('image_categories as categoryTable','categoryTable.image_id', '=', 'service.upload_image')
            ->leftJoin('image_categories as iconTable','iconTable.image_id', '=', 'service.upload_proof')
             ->select('service.service_id as id','category.category_name as category_name','service.service_status as service_status', 'service.name as name','service.mobile as mobile', 'service.upload_image as image','service.upload_proof as icon',  'service.created_at as date_added','service.updated_at as last_modified','service.email as email',
               'service.shift_start as shift_start','service.shift_end as shift_end','service.qualification as qualification','service.experience as experience','service.password as password','categoryTable.path as imgpath','iconTable.path as iconpath')
          
            ->where('service.service_id', $request->id)->get();
        return $editSubCategory;
    }


    public function editdescription($languages_id,$id){
        $description = DB::table('categories_description')->where([
            ['language_id', '=', $languages_id],
            ['categories_id', '=', $id],
        ])->get();
        return $description;
    }

   public function deleterecord($request){
        $service_id = $request->id;
        DB::table('service')->where('service_id', $service_id)->delete();
        return "success";
    }
    
    public function booking(){
      $setting = new Setting();
      $myVarsetting = new SiteSettingController($setting);
      $commonsetting = $myVarsetting->commonsetting();

      $categories =  DB::table('service_booking')
            ->select('service_booking.booking_id as id', 'service_booking.imgpath as imgpath','service_booking.first_name as first_name', 
            'service_booking.last_name as last_name', 'service_booking.category_name as category_name', 'service_booking.title as title','service_booking.price as price',
            'service_booking.description as description','service_booking.discount as discount','service_booking.time as time','service_booking.phone as phone','service_booking.email as email','service_booking.address as address','service_booking.full_name as full_name')
           ->groupby('service_booking.booking_id')
            ->paginate($commonsetting['pagination']);

            return $categories;
    }

}
