<?php

namespace App\Models\Core;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;
use Kyslik\ColumnSortable\Sortable;
use App\Http\Controllers\AdminControllers\SiteSettingController;

class ServiceProduct extends Model
{
    //
    use Sortable;
    public function images(){
        return $this->belongsTo('App\Images');
    }

    public function categories_description(){
        return $this->beliesngsTo('App\Categories_description');
    }

    public $sortable =['categories_id','created_at'];
    public $sortableAs =['categories_name'];

    public function recursivecategories(){
      $items = DB::table('categories')
          ->leftJoin('categories_description','categories_description.categories_id', '=', 'categories.categories_id')
          ->select('categories.categories_id', 'categories_description.categories_name', 'categories.parent_id')
          ->where('language_id','=', 1)
          ->where('categories_status', '1')
          //->orderby('categories_id','ASC')
          ->get();

          $childs = array();
          foreach($items as $item)
              $childs[$item->parent_id][] = $item;

          foreach($items as $item) if (isset($childs[$item->categories_id]))
              $item->childs = $childs[$item->categories_id];
          if(count($childs)>0){
            $tree = $childs[0];
          }else{
            $tree = $childs;
          }

          return  $tree;
    }

    public function editrecursivecategories($data){
      $items = DB::table('categories')
          ->leftJoin('categories_description','categories_description.categories_id', '=', 'categories.categories_id')
          ->select('categories.categories_id', 'categories_description.categories_name', 'categories.parent_id')
          ->where('language_id','=', 1)
          ->where('categories.categories_id','!=', $data->id)
          ->where('categories_status', '1')
          //->orderby('categories_id','ASC')
          ->get();

          $childs = array();
          foreach($items as $item)
              $childs[$item->parent_id][] = $item;

          foreach($items as $item) if (isset($childs[$item->categories_id]))
              $item->childs = $childs[$item->categories_id];

              if(count($childs)>0){
                  $tree = $childs[0];
                }else{
                  $tree = $childs;
                }
          return  $tree;
    }

    public function paginator(){
      $setting = new Setting();
      $myVarsetting = new SiteSettingController($setting);
      $commonsetting = $myVarsetting->commonsetting();

      $categories =  DB::table('service_products')
           ->LeftJoin('image_categories as categoryTable', function ($join) {
                $join->on('categoryTable.image_id', '=', 'service_products.image')
                    ->where(function ($query) {
                        $query->where('categoryTable.image_type', '=', 'THUMBNAIL')
                            ->where('categoryTable.image_type', '!=', 'THUMBNAIL')
                            ->orWhere('categoryTable.image_type', '=', 'ACTUAL');
                    });
            })
            ->leftJoin('category','service_products.category_id', '=', 'category.category_id')
            ->select('service_products.products_id as id', 'service_products.image as image',  'service_products.created_at as date_added', 'service_products.updated_at as last_modified',
            'service_products.title as title','service_products.price as price','service_products.description as description',
            'service_products.discount as discount','service_products.time as time','categoryTable.path as imgpath', 
            'service_products.service_status  as service_status', 'category.category_name as category_name')
         
            
            ->groupby('service_products.products_id')
            ->paginate($commonsetting['pagination']);

            return $categories;
    }



    
      public function getter(){
        
      $newsCategories = DB::table('category')
         // ->leftJoin('service_products','service_products.category_id', '=', 'category.category_id')
          ->select('category.category_id as id', 'category.category_name as category_name')
          ->get();

       return $newsCategories;
    }
    public function allcategories($language_id){
        $listingCategories = DB::table('categories')
            ->leftJoin('categories_description','categories_description.categories_id', '=', 'categories.categories_id')
            ->select('categories.categories_id as id', 'categories.categories_image as image',  'categories.created_at as date_added', 'categories.updated_at as last_modified', 'categories_description.categories_name as name', 'categories.categories_slug as slug')
            ->where('categories_description.language_id','=', $language_id )
            ->where('categories_status', '1')
            ->get();
  
         return $listingCategories;
      }

  


    public function insert($image,$date_added,$service_status,$title,$price,$discount,$description,$time,$category_id){
        $categories = DB::table('service_products')->insertGetId([
            'image'   =>   $image,
            'created_at'		 =>   $date_added,
            'title' => $title,
            'price' => $price,
            'discount' =>$discount,
            'description' => $description,
            'time' => $time,
            'service_status' => $service_status,
            'category_id' => $category_id,
        ]);
        return $categories;
    }



    public function edit($request){
      //  exit;
        $category = DB::table('service_products') ->leftJoin('images','images.id', '=', 'service_products.image')
            ->leftJoin('image_categories as ImageTable','ImageTable.image_id', '=', 'service_products.image')
           >select('service_products.products_id as id', 'service_products.image as image',  'service_products.created_at as date_added', 'service_products.updated_at as last_modified',
            'service_products.title as title','service_products.price as price','service_products.description as description',
            'service_products.discount as discount','service_products.time as time','categoryTable.path as imgpath', 
            'service_products.service_status  as service_status')
            ->where('service_products.products_id', $request->id)->get();
            //echo '<pre>';print_r($category);die;
        return $category;
    }
    
    public function listcategory($request){

        $listcategory = DB::table('category')
            //->leftJoin('category','service_products.category_id', '=', 'category.category_id')
            ->select('category.category_id as category_id','category.category_name as category_name')
            ->get();
            //echo'<pre>';print_r($listcategory);die;
            
        return $listcategory;
    }
    

    public function updaterecord($products_id,$image,$last_modified,$service_status,$title,$price,$discount,$description,$time,$category_id){
      DB::table('service_products')->where('products_id', $products_id)->update(
      [
          'image'   =>   $image,
          'updated_at'  	     =>   $last_modified,
           'title' => $title,
            'price' => $price,
            'discount' =>$discount,
            'description' => $description,
            'time' => $time,
            'service_status' => $service_status,
            'category_id' => $category_id,
      ]);
     
      
    }




    public function editsubcategory($request){
        
        $editSubCategory = DB::table('service_products')
        ->leftJoin('category','service_products.category_id', '=', 'category.category_id')
            ->leftJoin('image_categories as categoryTable','categoryTable.image_id', '=', 'service_products.image')
            ->select('service_products.products_id as id', 'service_products.image as image',  'service_products.created_at as date_added', 'service_products.updated_at as last_modified',
            'service_products.title as title','service_products.price as price','service_products.description as description',
            'service_products.discount as discount','service_products.time as time','categoryTable.path as imgpath', 
            'service_products.service_status  as service_status', 'category.category_name as category_name','category.category_id')
          
            ->where('service_products.products_id', $request->id)->get();
            //echo '<pre>';print_r($editSubCategory);die;
        return $editSubCategory;
    }




   public function deleterecord($request){
        $products_id = $request->id;
        DB::table('service_products')->where('products_id', $products_id)->delete();
        return "success";
    }

}
